package trains.model;

import java.util.GregorianCalendar;
import java.util.Locale;

public enum DayOfWeek
{
	SUNDAY(GregorianCalendar.SUNDAY, 7),
	MONDAY(GregorianCalendar.MONDAY, 1),
	TUESDAY(GregorianCalendar.TUESDAY, 2),
	WEDNESDAY(GregorianCalendar.WEDNESDAY, 3),
	THURSDAY(GregorianCalendar.THURSDAY, 4),
	FRIDAY(GregorianCalendar.FRIDAY, 5),
	SATURDAY(GregorianCalendar.SATURDAY, 6);	
	
	private int gregorianCalendarDay;
	private int trainDay;
	
	private DayOfWeek(int gregorianCalendarDay, int trainDayOfWeek)
	{
		this.gregorianCalendarDay = gregorianCalendarDay;
		this.trainDay = trainDayOfWeek;
	}	
	
	public int toGregorianCalendarDay()
	{
		return gregorianCalendarDay;
	}
	
	public int toTrainDay()
	{
		return trainDay;
	}
	
	@Override
	public String toString()
	{
		//Soluzione ufficiale
//		switch (this)
//		{
//			case SUNDAY: return "Sunday";
//			case MONDAY: return "Monday";
//			case TUESDAY: return "Tuesday";
//			case WEDNESDAY: return "Wednesday";
//			case THURSDAY: return "Thursday";
//			case FRIDAY: return "Friday";
//			case SATURDAY: return "Saturday";
//		}
		
		//Soluzione alternativa
		GregorianCalendar cal = new GregorianCalendar();
		cal.set(GregorianCalendar.DAY_OF_WEEK, gregorianCalendarDay);
		return cal.getDisplayName(GregorianCalendar.DAY_OF_WEEK, GregorianCalendar.LONG, Locale.ENGLISH);
	}
	
	public static DayOfWeek fromGregorianCalendarDay(int gregorianCalendarDay)
	{
		for (DayOfWeek day : DayOfWeek.values())
		{
			if (day.gregorianCalendarDay == gregorianCalendarDay)
			{
				return day;
			}
		}
		return null;
	}
	
	public static DayOfWeek fromTrainDay(int trainDay)
	{
		for (DayOfWeek day : DayOfWeek.values())
		{
			if (day.trainDay == trainDay)
			{
				return day;
			}
		}
		return null;
	}
}
