package trains.model;

import java.util.*;

public class Train
{
	private String type;
	private String number;
	private ArrayList<DayOfWeek> days;
	private SortedSet<Stop> stops;

	public Train(String type, String number)
	{
		this.type = type;
		this.number = number;
		days = new ArrayList<DayOfWeek>();
		stops = new TreeSet<Stop>();
	}

	public Time travelDuration(Station start, Station stop) throws InvalidStationException
	{
		if ((start == null) || (stop == null))
		{
			throw new InvalidStationException("Invalid parameter");
		}
		Time tStart = null;
		Time tEnd = null;

		for (Stop st : stops)
		{
			if (tStart == null)
			{
				if (st.getStation().compareTo(start) == 0)
				{
					tStart = st.getDeparture();
				}
			}
			else
			{
				if (st.getStation().compareTo(stop) == 0)
				{
					tEnd = st.getArrival();
					break;
				}
			}
		}

		if ((tStart == null) || (tEnd == null))
		{
			throw new InvalidStationException("The requested travel is not valid for this train");
		}

		int minutes = tEnd.getTotalMinutes() - tStart.getTotalMinutes();

		if (minutes < 0)
		{
			throw new InvalidStationException("The requested travel is not valid for this train");
		}
		return new Time(minutes / 60, minutes % 60);
	}

	public String getType()
	{
		return type;
	}

	public String getNumber()
	{
		return number;
	}

	public ArrayList<DayOfWeek> getDays()
	{
		return days;
	}

	public void setDays(ArrayList<DayOfWeek> days)
	{
		this.days = days;
	}

	public SortedSet<Stop> getStops()
	{
		return stops;
	}

	public void setStops(SortedSet<Stop> stops)
	{
		this.stops = stops;
	}

	@Override
	public String toString()
	{
		return (type.length() == 1 ? type + " " : type) + number;
	}

}
