package trains.persistence.text;

import java.util.SortedSet;
import trains.model.*;
import trains.persistence.BadFileFormatException;
import trains.persistence.binary.CityStationsReader;

import java.util.List;
import java.io.BufferedReader;
import java.io.IOException;
import java.io.Reader;
import java.util.ArrayList;
import java.util.StringTokenizer;
import java.util.TreeSet;

public class MyTrainReader implements TrainReader
{
	private List<Train> trains = null;

	public List<Train> readAll(Reader reader, CityStationsReader cities) throws IOException, BadFileFormatException
	{
		BufferedReader bufferedReader = new BufferedReader(reader);
		trains = new ArrayList<Train>();
		Train t;
		while ((t = readTrain(bufferedReader, cities)) != null)
		{
			trains.add(t);
		}

		bufferedReader.close();
		return trains;
	}

	private Train readTrain(BufferedReader reader, CityStationsReader cities) throws IOException, BadFileFormatException
	{
		Train train = null;
		String line = reader.readLine();
		if (line == null || line.trim().isEmpty())
			return null;
		StringTokenizer str = new StringTokenizer(line, " ");
		str.nextToken();
		String type = str.nextToken("0123456789").trim();
		String id = str.nextToken(",").trim();
		String day = str.nextToken().trim();
		train = new Train(type, id);
		train.setDays(readDays(day));
		train.setStops(readStops(reader, cities));
		return train;

	}

	private SortedSet<Stop> readStops(BufferedReader reader, CityStationsReader cities) throws IOException,
			BadFileFormatException
	{
		TreeSet<Stop> stops = new TreeSet<Stop>();
		String line = reader.readLine();
		while (!line.equals("#ENDTRAIN"))
		{
			StringTokenizer str = new StringTokenizer(line, ",");
			Station staz = readStation(str.nextToken().trim(), cities);
			Time arr = readTime(str.nextToken().trim());
			Time dep = readTime(str.nextToken().trim());
			stops.add(new Stop(staz, arr, dep));
			line = reader.readLine();
		}
		return stops;
	}

	private Station readStation(String stat, CityStationsReader cities) 
		throws BadFileFormatException
	{		
		City city = cities.getCity(stat);
		if (city == null)
			throw new BadFileFormatException("Stazione sconosciuta: " + stat);
		
		SortedSet<Station> stations = cities.getStations(city);
		for (Station station : stations)
		{
			String fullStationName = station.toString().trim();
			if (fullStationName.equalsIgnoreCase(stat))
			{
				return station;
			}
		}
		return null;	//Non si dovrebbe verificare mai...

//		Versione alternativa funzionante anche senza patch a CityStationsMock
//		Station station;
//		stat = stat.toUpperCase();
//		if (city.getName().equalsIgnoreCase(stat))
//		{
//			station = new Station(stat, "");
//		}
//		else
//		{			
//			String name = stat.replaceFirst(city.getName().toUpperCase(), "").trim();
//			station = new Station(city.getName(), name);
//		}
//		return station;
	}

	private Time readTime(String time) throws BadFileFormatException
	{
		if ("--:--".equals(time))
		{
			return null;
		}

		StringTokenizer tok = new StringTokenizer(time, ":");
		String hoursString = tok.nextToken().trim();
		String minutesString = tok.nextToken().trim();

		try
		{
			int hours = Integer.parseInt(hoursString);
			int minutes = Integer.parseInt(minutesString);
			return new Time(hours, minutes);
		}
		catch (NumberFormatException e)
		{
			throw new BadFileFormatException(e);
		}
	}

	private ArrayList<DayOfWeek> readDays(String days) throws BadFileFormatException
	{
		if (days.length() != 7)
			throw new BadFileFormatException();

		ArrayList<DayOfWeek> daysOfWeek = new ArrayList<DayOfWeek>();
		for (int i = 0; i < 7; i++)
		{
			char c = days.charAt(i);
			if (c == '1' + i)
			{
				daysOfWeek.add(DayOfWeek.fromTrainDay(c - '0'));
			}
			else if (c != '-')
			{
				throw new BadFileFormatException();
			}
		}
		return daysOfWeek;
	}
}
