package trains.tests;
import java.io.IOException;
import java.io.StringReader;
import java.util.ArrayList;

import junit.framework.Assert;

import org.junit.Test;

import trains.model.Train;
import trains.persistence.BadFileFormatException;
import trains.persistence.InvalidDataException;
import trains.persistence.text.MyTrainReader;
public class MyTrainReaderTest {


	@Test
	public void testReadAll() throws IOException, InvalidDataException, BadFileFormatException {
		String dati =
			"#TRAIN R3961, 1234567\n" +
			"Bologna centrale,9:28, 9:36\n" +
			"Castel S.Pietro,11:05,--:--\n" +
			"#ENDTRAIN\n" +
			"#TRAIN IR2069,12345--\n" +
			"Bologna centrale,--:--,10:36\n" +
			"Castel S.Pietro, 10:51,10:52\n" +
			"Rimini,12:30,--:--\n" +
			"#ENDTRAIN\n";

		CityStationsMock mock = new CityStationsMock();

		MyTrainReader trainReader = new MyTrainReader();
		ArrayList<Train> schedules = (ArrayList<Train>) trainReader.readAll(new StringReader(dati), mock);

		Assert.assertEquals(2, schedules.size());
	    Assert.assertTrue(schedules.get(0).getNumber().equals("3961"));
		Assert.assertTrue(schedules.get(1).getNumber().equals("2069"));
	
		Assert.assertEquals(2, schedules.get(0).getStops().size());
		Assert.assertEquals(3, schedules.get(1).getStops().size());

		Assert.assertTrue(schedules.get(0).getStops().first().getStation().getStationName().equalsIgnoreCase("CENTRALE"));
		Assert.assertTrue(schedules.get(0).getStops().first().getStation().getCityName().equalsIgnoreCase("BOLOGNA"));
		Assert.assertEquals(9, schedules.get(0).getStops().first().getArrival().getHours());
		Assert.assertEquals(28, schedules.get(0).getStops().first().getArrival().getMinutes());
		Assert.assertEquals(9, schedules.get(0).getStops().first().getDeparture().getHours());
		Assert.assertEquals(36, schedules.get(0).getStops().first().getDeparture().getMinutes());

		Assert.assertTrue(schedules.get(0).getStops().last().getStation().getCityName().equalsIgnoreCase("CASTEL S.PIETRO"));
		Assert.assertNull(schedules.get(0).getStops().last().getDeparture());

		Assert.assertTrue(schedules.get(1).getStops().first().getStation().getStationName().equalsIgnoreCase("CENTRALE"));
		Assert.assertTrue(schedules.get(1).getStops().last().getStation().getStationName().equalsIgnoreCase(""));
		Assert.assertNull(schedules.get(1).getStops().first().getArrival());
		Assert.assertNull(schedules.get(1).getStops().last().getDeparture());
	}

	private void checkParseException(String dati) throws IOException, InvalidDataException
	{
		CityStationsMock mock = new CityStationsMock();

		boolean ok = false;
		MyTrainReader trainReader = new MyTrainReader();
		try
		{
			trainReader.readAll(new StringReader(dati), mock);
		}
		catch (BadFileFormatException e)
		{
			ok = true;
		}
		Assert.assertTrue(ok);
	}

	@Test
	public void testReadAllWrongFile() throws IOException, InvalidDataException {
		checkParseException(
				"#TRAIN R3961,1234567\n" +
				"Bologna centrale,A:28, 9:36\n" +
				"Castel S.Pietro,11:05,--:--\n" +
				"#ENDTRAIN\n" +
				"#TRAIN IR2069,12345--\n" +
				"Bologna centrale,--:--,10:36\n" +
				"Castel S.Pietro,10:51,10:52\n" +
				"Rimini,12:30,--:--\n" +
				"#ENDTRAIN\n"
		);

		checkParseException(
				"#TRAIN R3961,1234567\n" +
				"Bologna centrale,9:28,9:36\n" +
				"Castel S.Pietro,--:05,--:--\n" +
				"#ENDTRAIN\n" +
				"#TRAIN IR2069,12345--\n" +
				"Bologna centrale,--:--,10:36\n" +
				"Castel S.Pietro,10:51,10:52\n" +
				"Rimini,12:30,--:--\n" +
				"#ENDTRAIN\n"
			);

		checkParseException(
				"#TRAIN R3961,1234567\n" +
				"Bologna centrale,A:28,9:36\n"
			);

		checkParseException(
				"Bologna centrale,A:28,9:36\n" +
				"Castel S.Pietro,11:05,--:--\n" +
				"#ENDTRAIN\n"
			);
	}
}
