package trains.ui;

import java.io.IOException;
import java.io.Reader;
import java.util.Date;
import java.util.GregorianCalendar;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.SortedSet;

import trains.model.*;
import trains.persistence.BadFileFormatException;
import trains.persistence.binary.*;
import trains.persistence.text.*;

public class MyController implements Controller
{

	private Set<String> sortedCities;
	private CityStationsReader cities;
	private Reader reader;
	private TrainReader readTrain;
	private List<Train> trains;

	public MyController(CityStationsReader cities, Reader reader) throws IOException, BadFileFormatException
	{
		this.cities = cities;
		this.readTrain = new MyTrainReader();
		this.reader = reader;
		this.sortedCities = cities.getMap().keySet();
		trains = readTrain.readAll(this.reader, this.cities);
	}

	public Set<String> getSortedCities()
	{
		return sortedCities;
	}

	public Map<Train, Time> searchTrains(Station departureStation, Station arrivalStation, Date date)
	{
		GregorianCalendar calendar = new GregorianCalendar();
		calendar.setTime(date);
		DayOfWeek dayOfWeek = DayOfWeek.fromGregorianCalendarDay(calendar.get(GregorianCalendar.DAY_OF_WEEK));
		HashMap<Train, Time> availableTrains = new HashMap<Train, Time>();

		for (Train t : trains)
		{
			Time time;
			if (t.getDays().contains(dayOfWeek) && areDepartureArrivalPresent(t.getStops(), departureStation, arrivalStation))
			{
				try
				{
					time = t.travelDuration(departureStation, arrivalStation);
					availableTrains.put(t, time);
				}
				catch (InvalidStationException e)
				{
					e.printStackTrace();
					// Non si deve MAI verificare!!!
				}
			}

		}
		return availableTrains;
	}

	private boolean areDepartureArrivalPresent(SortedSet<Stop> stops, Station departure, Station arrival)
	{
		// La partenza deve essere PRIMA dell'arrivo!
		boolean departureFound = false;
		for (Stop stop : stops)
		{
			if (!departureFound && stop.getStation().equals(departure))
			{
				departureFound = true;
			}
			else if (stop.getStation().equals(arrival))
			{
				return departureFound;
			}
		}
		return false;
	}

}
