package trains.model;

public class Time {
		private int hours;
		private int minutes;
		
		public static final Time MIN_VALUE = new Time(0, 0); 
		public static final Time MAX_VALUE = new Time(23, 59);
		
		public Time(int hours, int minutes)
		{
			this.hours = hours % 24;
			this.minutes = minutes % 60;
		}
		
		public int getHours()
		{
			return hours;
		}
		
		public int getMinutes()
		{
			return minutes;
		}
		
		public int getTotalMinutes()
		{
			return hours * 60 + minutes;
		}
		
		public Time addMinutes(int mins)
		{
			int newHours;
			int newMinutes;
			
			newMinutes = this.minutes + mins;
			newHours = this.hours + (newMinutes / 60);
			newMinutes %= 60;
			if (newMinutes < 0)
			{
				newHours--;
				newMinutes += 60;
			}
			newHours %= 24;
			if (newHours < 0)
			{
				newHours += 24;
			}
			return new Time(newHours, newMinutes);
		}	
		
		public boolean isValid()
		{
			return this.compareTo(MIN_VALUE) >= 0 &&
				this.compareTo(MAX_VALUE) <= 0;
		}
		
		@Override
		public boolean equals(Object obj)
		{
			if (obj == null || !(obj instanceof Time))
			{
				return false;
			}
			Time time = (Time)obj;
			return this.hours == time.hours && this.minutes == time.minutes;
		}
		
		public int compareTo(Time time)
		{
			if (this.hours > time.hours)
			{
				return 1;
			}
			else if (this.hours == time.hours) 
			{
				if (this.minutes > time.minutes)
				{
					return 1;
				}
				else if (this.minutes == time.minutes)
				{
					return 0;
				}
				else
				{
					return -1;
				}
			}
			else 
			{
				return -1;
			}
		}
		
		@Override
		public String toString()
		{
			return "" + hours + ":" + (minutes<10 ? "0"+minutes : minutes);
		}
	}

